@@ -19,6 +19,7 @@ import errno
 import socket
 import sys
 import time
+import urllib3
 from urllib3.exceptions import HTTPError
 
 class DeploymentOrchestrator(object):
@@ -37,7 +38,7 @@ class DeploymentOrchestrator(object):
     def ping(self):
         try:
             return bool(self.etcd.machines)
-        except HTTPError:
+        except etcd.EtcdError, HTTPError:
             return False
 
     def update_own_info(self, hostname, interval=60, version=None):
@@ -51,14 +52,21 @@ class DeploymentOrchestrator(object):
         return filter(lambda x: x != 'running_version',
                       [x.key.split('/')[-1] for x in res.children])
 
-    def check_single_version(self, version):
+    def check_single_version(self, version, verbose=False):
         desired_version_seen = False
-        for v in self.running_versions():
-            if v != version:
-                return False
-            else:
-                desired_version_seen = True
-        return desired_version_seen
+        running_versions = self.running_versions()
+        unwanted_versions = filter(lambda x:x != version,
+                                   running_versions)
+        wanted_version_found = version in running_versions
+        if verbose:
+            print 'Wanted version found:', wanted_version_found
+            print 'Unwanted versions found:', ', '.join(unwanted_versions)
+        return wanted_version_found and not unwanted_versions
+
+    def new_discovery_token(self, discovery_endpoint):
+        http = urllib3.PoolManager()
+        r = http.request('GET', '%snew' % (discovery_endpoint,))
+        return r.data.split('/')[-1]
 
     def local_version(self, new_value=None):
         mode = new_value is None and 'r' or 'w'
@@ -101,8 +109,12 @@ if __name__ == '__main__':
                                         help="Override version to report into etcd")
     running_versions_parser = subparsers.add_parser('running_versions', help="List currently running versions")
 
+    new_discovery_token_parser = subparsers.add_parser('new_discovery_token', help="Get new discovery token")
+    new_discovery_token_parser.add_argument('--endpoint', default='https://discovery.etcd.io/', help='Discovery token endpoint')
+
     check_single_version_parser = subparsers.add_parser('check_single_version', help="Check if the given version is the only one currently running")
     check_single_version_parser.add_argument('version', help='The version to check for')
+    check_single_version_parser.add_argument('--verbose', '-v', action='store_true', help='Be verbose')
     args = parser.parse_args()
 
     do = DeploymentOrchestrator(args.host, args.port)
@@ -111,7 +123,7 @@ if __name__ == '__main__':
     elif args.subcmd == 'current_version':
         print do.current_version()
     elif args.subcmd == 'check_single_version':
-        print do.check_single_version(args.version)
+        sys.exit(not do.check_single_version(args.version, args.verbose))
     elif args.subcmd == 'update_own_info':
         do.update_own_info(args.hostname, version=args.version)
     elif args.subcmd == 'ping':
@@ -126,6 +138,8 @@ if __name__ == '__main__':
         print do.local_version(args.version)
     elif args.subcmd == 'running_versions':
         print '\n'.join(do.running_versions())
+    elif args.subcmd == 'new_discovery_token':
+        print do.new_discovery_token(args.endpoint)
     elif args.subcmd == 'pending_update':
         pending_update = do.pending_update()
         if pending_update:
